package net.sf.seaf.service;

import java.util.HashMap;

import net.sf.seaf.exception.NotInitializedException;
import net.sf.seaf.factory.Factory;
import net.sf.seaf.service.sit.CodeTableRefreshingConverter;
import net.sf.sit.SITConfigurer;
import net.sf.sit.candidate.impl.ConfigurableMatcher;
import net.sf.sit.candidate.impl.MatchingSelector;
import net.sf.sit.candidate.impl.MethodNameMatcher;
import net.sf.sit.exception.ExceptionHandler;

import org.dozer.Mapper;

/**
 * Abstract base class of service factories.
 * <p>
 * Provides common functionality for service factories: creating
 * {@link SITConfigurer}, configuring its method overrides and instantiating
 * server controllers and setting them as providers to the SITConfigurer.
 */
public abstract class ServiceFactoryBase {

	/**
	 * Default constructor.
	 */
	public ServiceFactoryBase() {
	}

	/**
	 * Full constructor.
	 * 
	 * @param persistenceFactory
	 *            Server controllers' factory
	 */
	public ServiceFactoryBase(Factory persistenceFactory) {
		this.persistenceFactory = persistenceFactory;
	}

	// support methods

	/**
	 * Create {@link SITConfigurer} based on the service facade type and types
	 * of controller providers.
	 * 
	 * @param serviceType
	 *            Service facade type
	 * @param controllerTypes
	 *            Types of controller providers
	 * @return SITConfigurer
	 */
	protected SITConfigurer createSITConfigurer(Class<?> serviceType,
			Class<? extends ServiceController>... controllerTypes) {
		SITConfigurer sc = newSITConfigurer();
		sc.setDozer(dozer);
		sc.addToDozerMappingFileNames("seaf-service-dozer-configuration.xml");
		sc.setFacadeType(serviceType);
		setProviders(sc, controllerTypes);
		setOverrides(sc);
		sc.setExceptionHandler(exceptionHandler);
		sc.init();
		sc.setArgumentsConverter(new CodeTableRefreshingConverter(sc
				.getArgumentsConverter(), getPersistenceFactory()));
		return sc;
	}

	/**
	 * Instantiation of SITConfigurer extracted into separate method to
	 * facilitate testing.
	 * 
	 * @return New instance of SITConfigurer
	 */
	protected SITConfigurer newSITConfigurer() {
		return new SITConfigurer();
	}

	/**
	 * Set method overrides on SITConfigurer.
	 * 
	 * @param sc
	 *            SITConfigurer
	 */
	private void setOverrides(SITConfigurer sc) {
		HashMap<String, String> overrides = new HashMap<String, String>();
		setMethodOverrides(overrides);
		sc.setSelector(new MatchingSelector(new ConfigurableMatcher(overrides,
				new MethodNameMatcher())));
	}

	/**
	 * Set providers on SITConfigurer.
	 * <p>
	 * Instantiates server controllers by their types and sets them as providers
	 * to the SITConfigurer.
	 * 
	 * @param sc
	 *            SITConfigurer
	 * @param controllerTypes
	 *            Server controller types
	 */
	private void setProviders(SITConfigurer sc,
			Class<? extends ServiceController>... controllerTypes) {
		for (int i = 0; i < controllerTypes.length; i++) {
			ServiceController controller = getPersistenceFactory()
					.getInstanceOf(controllerTypes[i]);
			sc.addToProviders(controller);
		}
	}

	// abstract strategy methods

	/**
	 * Provide method overrides in form of a HashMap.
	 * <p>
	 * Abstract strategy method to be implemented by specific service factories.
	 * 
	 * @param overrides
	 *            Method overrides
	 */
	protected abstract void setMethodOverrides(HashMap<String, String> overrides);

	// standard getters / setters

	/**
	 * Factory used to instantiate server controllers.
	 */
	private Factory persistenceFactory;

	/**
	 * Return server controllers' factory.
	 * 
	 * @return Server controllers' factory
	 */
	public final Factory getPersistenceFactory() {
		if (null == persistenceFactory)
			throw new NotInitializedException(
					"Persistence factory was not initialized.");
		return persistenceFactory;
	}

	/**
	 * Set server controllers' factory.
	 * 
	 * @param persistenceFactory
	 *            Server controllers' factory
	 */
	public final void setPersistenceFactory(Factory persistenceFactory) {
		this.persistenceFactory = persistenceFactory;
	}

	/**
	 * Dozer {@link Mapper}.
	 */
	private Mapper dozer;

	/**
	 * Set Dozer {@link Mapper}.
	 * 
	 * @param dozer
	 *            Dozer Mapper
	 */
	public void setDozer(Mapper dozer) {
		this.dozer = dozer;
	}

	/**
	 * Exception handler.
	 */
	private ExceptionHandler exceptionHandler;

	/**
	 * Set exception handler.
	 * 
	 * @param exceptionHandler
	 *            Exception handler
	 */
	public void setExceptionHandler(ExceptionHandler exceptionHandler) {
		this.exceptionHandler = exceptionHandler;
	}

}
