package net.sf.seaf.service.sit;

import net.sf.seaf.factory.Factory;
import net.sf.seaf.persistence.Entity;
import net.sf.seaf.persistence.util.CodeTablePropertyRefresher;
import net.sf.sit.values.Converter;

/**
 * SIT converter that converts the value using a backing converter and then
 * applies {@link CodeTablePropertyRefresher} on the value before returning it.
 * <p>
 * It is meant to be applied as arguments converter in SIT used for service
 * layer, where it refreshes all code table-typed properties on entity before
 * passing it into service controller.
 * <p>
 * Values that are not {@link Entity} bypass the refresh.
 */
public class CodeTableRefreshingConverter implements Converter {

	private final Converter backingConverter;
	private final Factory controllerFactory;

	/**
	 * Standard constructor.
	 * 
	 * @param backingConverter
	 *            Backing converter, used to convert the value from service
	 *            object to entity before applying
	 *            {@link CodeTablePropertyRefresher}
	 * @param controllerFactory
	 *            Factory that provides code table controllers
	 */
	public CodeTableRefreshingConverter(Converter backingConverter,
			Factory controllerFactory) {
		this.backingConverter = backingConverter;
		this.controllerFactory = controllerFactory;
	}

	public <TT> TT convert(Class<TT> targetType, Object sourceValue,
			Class<?> targetComponentTypeHint) {
		TT converted = backingConverter.convert(targetType, sourceValue,
				targetComponentTypeHint);
		if (null != converted
				&& Entity.class.isAssignableFrom(converted.getClass())) {
			getRefresher().setControllerFactory(controllerFactory);
			getRefresher().refresh((Entity<?>) converted);
		}
		return converted;
	}

	private CodeTablePropertyRefresher refresher;

	/**
	 * Return refresher.
	 * 
	 * @return the refresher
	 */
	public CodeTablePropertyRefresher getRefresher() {
		if (null == refresher) {
			refresher = new CodeTablePropertyRefresher();
		}
		return refresher;
	}

	/**
	 * Set refresher.
	 * 
	 * @param refresher
	 *            the refresher to set
	 */
	public void setRefresher(CodeTablePropertyRefresher refresher) {
		this.refresher = refresher;
	}

	/**
	 * Return the backing converter.
	 * 
	 * @return The backingConverter
	 */
	public Converter getBackingConverter() {
		return backingConverter;
	}

	/**
	 * Return the controller factory.
	 * 
	 * @return The controllerFactory
	 */
	public Factory getControllerFactory() {
		return controllerFactory;
	}

}
